import os.path as osp


def main(**kwargs):
    """
    Will be called after build.
    kwargs:
        proj_root: project root path
        build_config: build configuration (Debug, Profile, Release)
        plugin_name: plugin name, which is used to glob plugin files
    """


if __name__ == '__main__':
    _proj_root = osp.abspath(f'{__file__}/../../..')
    _plugin_name = osp.basename(_proj_root)
    # Just for manually run
    main(proj_root=_proj_root, build_config='Debug', plugin_name=_plugin_name)



import logging
import os.path as osp
import platform

import kkpyutil as util

# project
from wpe.util import *
from wpe.pathman import PathMan


class PluginInfo:
    def __init__(self, info_dict: dict):
        self.infoDict = info_dict
        self.defaultPlatformSupport = '''<Platform Name="Any">
  <CanBeInsertOnBusses>true</CanBeInsertOnBusses>
  <CanBeInsertOnAudioObjects>true</CanBeInsertOnAudioObjects>
  <CanBeRendered>true</CanBeRendered>
</Platform>'''

    def generate_platform_support(self) -> list[str]:
        if not self.infoDict['platform_support']:
            return self.defaultPlatformSupport.splitlines()
        lines = []
        for platform, config in self.infoDict['platform_support'].items():
            lines.append(f'<Platform Name="{platform}">')
            for key, value in config.items():
                lines.append(f'  <{key}>{str(value).lower()}</{key}>')
            lines.append('</Platform>')
        return lines


class PlatformTarget:
    def __init__(self, target_dict: dict):
        self.targetDict = target_dict
        self.platform: str = target_dict['platform']
        self.architectures: list[str] = target_dict['architectures']

    def is_windows(self) -> bool:
        return self.platform.startswith('Windows')

    def is_authoring(self) -> bool:
        return self.platform == 'Authoring'

    def need_toolset(self) -> bool:
        return self.is_windows() or self.is_authoring()

    def toolset(self):
        if ts := self.targetDict.get('toolset'):
            return ts
        return self.platform.split('_')[1] if self.is_windows() else None


class ProjectConfig:
    def __init__(self, path_man: PathMan):
        self.pathMan = path_man
        self.config = None
        self.load()

    def load(self):
        if osp.isfile(self.pathMan.projConfig):
            self.config = load_toml(self.pathMan.projConfig)
            return

        if osp.isfile(self.pathMan.parameterConfig):
            self.config = load_toml(self.pathMan.parameterConfig)
            logging.warning('wpe_parameters.toml is deprecated. Please rename it to wpe_project.toml.')
            return

        raise FileNotFoundError(f'wpe project config not found: {self.pathMan.projConfig}')

    def bump(self):
        new_version = self.version() + 1
        config_lines = util.load_lines(self.pathMan.projConfig)
        for i, line in enumerate(config_lines):
            if line.startswith('version = '):
                config_lines[i] = f'version = {new_version}\n'
                break
        util.save_lines(self.pathMan.projConfig, config_lines)
        self.load()

    def target_platforms(self) -> list[PlatformTarget]:
        platform_targets_key = 'win_targets' if platform.system() == 'Windows' else 'mac_targets'
        return [PlatformTarget(target) for target in self.config['project'][platform_targets_key]]

    def all_platforms(self) -> list[str]:
        return list(set([PlatformTarget(target).platform for target in self.config['project']['win_targets'] + self.config['project']['mac_targets']]))

    def plugin_info(self) -> PluginInfo:
        return PluginInfo(self.config['plugin_info'])

    def has_parameters(self) -> bool:
        return 'parameters' in self.config

    def parameter_defines(self) -> dict:
        return self.config['parameters'].get('defines', {})

    def parameter_templates(self) -> dict:
        return self.config['parameters'].get('templates', {})

    def parameter_inner_types(self) -> dict:
        return self.config['parameters'].get('inner_types', {})

    def parameter_from_templates(self) -> list:
        return self.config['parameters'].get('from_templates', [])

    def parameter_from_inner_types(self) -> list:
        return self.config['parameters'].get('from_inner_types', [])

    def version(self) -> int:
        return self.config['project']['version']



"""
The content of this file includes portions of the AUDIOKINETIC Wwise Technology
released in source code form as part of the SDK installer package.

Commercial License Usage

Licensees holding valid commercial licenses to the AUDIOKINETIC Wwise Technology
may use this file in accordance with the end user license agreement provided
with the software or, alternatively, in accordance with the terms contained in a
written agreement between you and Audiokinetic Inc.

  Copyright (c) 2021 Audiokinetic Inc.
"""

from __future__ import absolute_import
import glob
import multiprocessing
import os
import platform
import shutil
import subprocess
from common.constant import PLUGIN_NAME, WWISE_ROOT
from common.hook import invoke, POSTBUILD_HOOK
from common.registry import platform_registry

SUPPORTED_BUILD_SYSTEMS = ("Windows","Linux")

def build(target, config, hooks):
    platform_info = platform_registry.get(target)
    for arch in platform_info.build.archs:
        build_tool = os.path.join(os.environ["NDKROOT"], "ndk-build")
        # [wp-enhanced patch] use WWISESDK env var (symlinked) to get the Wwise SDK path to avoid space in path issue
        out_dir_root = os.path.join(os.getenv('WWISESDK'), "Android_" + arch)
        # [/wp-enhanced patch]
        application_mk = " NDK_APPLICATION_MK=" + PLUGIN_NAME + "_Android_application.mk"
        libs_out = " NDK_LIBS_OUT=" + os.path.join(out_dir_root, config, "libs").replace("\\", "/")
        ndk_out = " NDK_OUT=" + os.path.join(out_dir_root, config, "lib").replace("\\", "/") # the missing "s" at lib is on purpose
        ndk_app_out = " NDK_APP_OUT=" + out_dir_root.replace("\\", "/")
        target_out = " TARGET_OUT=" + os.path.join(out_dir_root, config, "lib").replace("\\", "/")

        if platform.system() == "Linux":
            ndk_project_path = "."
        else:
            ndk_project_path = ".\\"

        build_command = "{} all -j {} NDK_PROJECT_PATH={} PM5_CONFIG={}".format(
            build_tool,
            str(multiprocessing.cpu_count()),
            ndk_project_path,
            config.lower() + "_android_" + arch + application_mk + libs_out + ndk_out + ndk_app_out + target_out
        )

        print("Building {} in {} using ndk-build. Build Command:\n{}".format(target, config, build_command))
        res = subprocess.Popen(build_command, shell=True).wait()
        if res != 0:
            return res

        # need to copy dynamic lib to final destination
        so_dstdir = os.path.join(out_dir_root, config, "bin")
        if not os.path.exists(so_dstdir):
            os.makedirs(so_dstdir)

        so_glob = os.path.join(out_dir_root, config, "libs", arch, "lib*.so")
        for so_fname in glob.glob(so_glob):
            shutil.copy(so_fname, so_dstdir)

        invoke(POSTBUILD_HOOK, hooks, target, config, arch)

    return 0



"""
The content of this file includes portions of the AUDIOKINETIC Wwise Technology
released in source code form as part of the SDK installer package.

Commercial License Usage

Licensees holding valid commercial licenses to the AUDIOKINETIC Wwise Technology
may use this file in accordance with the end user license agreement provided
with the software or, alternatively, in accordance with the terms contained in a
written agreement between you and Audiokinetic Inc.

  Copyright (c) 2021 Audiokinetic Inc.
"""

from __future__ import absolute_import
import os
# [wp-enhanced patch] import android platform patch
from wpe.wp_patch.common.command.android import build, SUPPORTED_BUILD_SYSTEMS
# [/wp-enhanced patch]
from common.registry import PlatformInfo, PremakeInfo, BuildInfo, PackageInfo, platform_registry

platform_registry["Android"] = PlatformInfo(
    name="Android",
    premake=PremakeInfo(
        actions=("androidmk",)
    ),
    build=BuildInfo(
        command=build,
        configurations=("Debug", "Profile", "Release"),
        archs=("armeabi-v7a", "x86", "arm64-v8a", "x86_64"),
        on=SUPPORTED_BUILD_SYSTEMS,
        require_configuration=True
    ),
    package=PackageInfo(
        artifacts = [
            os.path.join("SDK", "Android_*", "*", "bin", "lib{plugin_name}.so"),
            os.path.join("SDK", "Android_*", "*", "lib", "lib{plugin_name}*.a"),
        ]
    )
)
